package com.iden.bms.service;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.iden.common.cache.RedisKeyConstant;
import com.iden.common.cache.RedisUtil;
import com.iden.common.entity.*;
import com.iden.common.enums.*;
import com.iden.common.exception.BDException;


import com.iden.common.service.*;
import com.iden.common.util.DateUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;


import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 *
 * @author makejava
 * @since 2021-05-21 00:08:38
 */
@Service
public class WarningService {

    @Resource
    private IdenFaceImageService idenFaceImageService;
    @Resource
    private IdenCameraService idenCameraService;
    @Resource
    private IdenCommunityService idenCommunityService;
    @Resource
    private IdenWarningEventService idenWarningEventService;
    @Resource
    private IdenWarningPersonService idenWarningPersonService;
    @Resource
    private IdenWarningPersonHisService idenWarningPersonHisService;
    @Resource
    private IdenWarningPersonTraceService idenWarningPersonTraceService;
    @Resource
    private IdenWarningStaService idenWarningStaService;
    @Resource
    private IdenWarningRuleStrangerService idenWarningRuleStrangerService;
    @Resource
    private IdenWarningRuleResidentService idenWarningRuleResidentService;
    @Resource
    private IdenWarningRuleEveningCommingService idenWarningRuleEveningCommingService;
    @Resource
    private RedisUtil redisUtil;
    @Resource
    private IdenPersonService idenPersonService;
    @Resource
    private IdenPersonStrangerService idenPersonStrangerService;

    @Value("${sta.before.days:#{null}}")
    private String staBeforeDays;

    /**
     * 实时处理轨迹、重点人员预警等
     * @return
     */

    public void handleActualTimeWarning() throws BDException {
      //查询没有处理过的人脸图像数据
        QueryWrapper<IdenFaceImage> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(IdenFaceImage::getHandleWarningStatus, HandleWarningStatusEnum.NO_HANDLE.getValue())
        .last("limit 5000"); //每次处理5000条
        List<IdenFaceImage> idenFaceImageList = idenFaceImageService.list(queryWrapper);
        if (CollUtil.isEmpty(idenFaceImageList)) {
            return;
        }

        for (IdenFaceImage idenFaceImage : idenFaceImageList) {

            //生成轨迹
            IdenWarningPersonTrace idenWarningPersonTrace = new IdenWarningPersonTrace();
            BeanUtil.copyProperties(idenFaceImage,idenWarningPersonTrace,"id");
            this.idenWarningPersonTraceService.save(idenWarningPersonTrace);

            String type = idenFaceImage.getType();
            //重点人员
            if (StringUtils.isNotEmpty(type) && type.contains(PersonTypeEnum.KEYNOTE.getValue())) {
                handleKeynote(idenFaceImage);
            }

            IdenFaceImage idenFaceImageUpdate = new IdenFaceImage();
            idenFaceImageUpdate.setId(idenFaceImage.getId());
            idenFaceImageUpdate.setHandleWarningStatus(HandleWarningStatusEnum.HANDLED.getValue());
            this.idenFaceImageService.updateById(idenFaceImageUpdate);
        }

    }

    //重点人员识别预警
    private void handleKeynote(IdenFaceImage idenFaceImage){
        Long communityId = idenFaceImage.getCommunityId();
        if(communityId == null) {
            return;
        }
        //查询是不是已经有预警人员数据了
        QueryWrapper<IdenWarningPerson> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda()
                .eq(IdenWarningPerson::getCommunityId,communityId)
                .eq(IdenWarningPerson::getUid,idenFaceImage.getUid())
                .eq(IdenWarningPerson::getType,WarningTypeEnum.KEYNOTE.getValue());
        IdenWarningPerson idenWarningPerson = this.idenWarningPersonService.getOne(queryWrapper);
        if (idenWarningPerson != null) {
            idenWarningPerson.setAppearFrequency(idenWarningPerson.getAppearFrequency() + 1);
            idenWarningPerson.setHandleStatus(HandleWarningStatusEnum.NO_HANDLE.getValue());
            idenWarningPerson.setModifyTime(new Date());
            this.idenWarningPersonService.updateById(idenWarningPerson);
        } else {
            idenWarningPerson = new IdenWarningPerson();
            idenWarningPerson.setUid(idenFaceImage.getUid());
            idenWarningPerson.setPersonId(idenFaceImage.getPersonId());
            idenWarningPerson.setImage(idenFaceImage.getImage());
            idenWarningPerson.setFeatPtr(idenFaceImage.getFeatPtr());
            idenWarningPerson.setDataDate(idenFaceImage.getDataDate());
            Long cameraId = idenFaceImage.getCameraId();
            if(cameraId != null) {
                IdenCamera idenCamera = this.idenCameraService.getById(cameraId);
                idenWarningPerson.setDistrict(idenCamera.getDistrict());
                idenWarningPerson.setSubdistrict(idenCamera.getSubdistrict());
            }
            IdenCommunity idenCommunity = this.idenCommunityService.getById(communityId);
            if (idenCommunity != null) {
                idenWarningPerson.setCommunityId(communityId);
                idenWarningPerson.setCommunityCode(idenCommunity.getCode());
                idenWarningPerson.setCommunityName(idenCommunity.getName());
            }

            idenWarningPerson.setAppearFrequency(1);

            //查询此人在该小区最早出现时间
            QueryWrapper<IdenFaceImage> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.lambda().eq(IdenFaceImage::getCommunityId,communityId).eq(IdenFaceImage::getUid,idenFaceImage.getUid())
                    .orderByAsc(IdenFaceImage::getPhotographTime);
            queryWrapper1.last("limit 1");
            IdenFaceImage idenFaceImage1 = this.idenFaceImageService.getOne(queryWrapper1);
            if(idenFaceImage1 != null){
                idenWarningPerson.setEarliestAppearTime(idenFaceImage1.getPhotographTime());
            }

            idenWarningPerson.setType(WarningTypeEnum.KEYNOTE.getValue());
            idenWarningPerson.setPersonId(idenFaceImage.getPersonId());
            if(idenWarningPerson.getPersonId() != null){
                IdenPerson idenPerson = this.idenPersonService.getById(idenWarningPerson.getPersonId());
                if(idenPerson != null){
                    idenWarningPerson.setPersonName(idenPerson.getName());
                }
            }
            idenWarningPerson.setCreateTime(new Date());
            this.idenWarningPersonService.save(idenWarningPerson);

        }
        IdenWarningPersonHis idenWarningPersonHis = new IdenWarningPersonHis();
        BeanUtil.copyProperties(idenWarningPerson,idenWarningPersonHis,"id");
        idenWarningPersonHis.setCreateTime(new Date());
        idenWarningPersonHisService.save(idenWarningPersonHis);

        IdenWarningEvent idenWarningEvent = new IdenWarningEvent();
        idenWarningEvent.setWarningPersonId(idenWarningPerson.getId());
        idenWarningEvent.setUid(idenWarningPerson.getUid());
        idenWarningEvent.setPersonId(idenWarningPerson.getPersonId());
        idenWarningEvent.setDataTime(idenFaceImage.getPhotographTime());
        idenWarningEvent.setImage(idenFaceImage.getImage());
        idenWarningEvent.setCommunityId(idenWarningPerson.getCommunityId());
        idenWarningEvent.setCommunityCode(idenWarningPerson.getCommunityCode());
        idenWarningEvent.setCommunityName(idenWarningPerson.getCommunityName());
        idenWarningEvent.setDistrict(idenWarningPerson.getDistrict());
        idenWarningEvent.setSubdistrict(idenWarningPerson.getSubdistrict());
        idenWarningEvent.setAppearTime(idenFaceImage.getPhotographTime());
        idenWarningEvent.setType(WarningTypeEnum.KEYNOTE.getValue());
        idenWarningEvent.setPersonId(idenFaceImage.getPersonId());
        idenWarningEvent.setCreateTime(new Date());
        this.idenWarningEventService.save(idenWarningEvent);

    }

    /**
     * 5分钟处理一次，处理前一个月的、按天的陌生人预警、常驻人员预警、昼伏夜出预警数据
     * @return
     */
    public void handleDayWarning() throws BDException {
        int staBeforeDayNums = Integer.parseInt(staBeforeDays);

        handleStranger();//陌生人员识别预警
        handleResident();//常住人员识别预警
        handleEveningComming();//昼伏夜出人员识别预警

        staCommunity(staBeforeDayNums);//按小区统n天前的数据
    }

    //陌生人员识别预警
    private void handleStranger() {
        List<IdenCommunity>  idenCommunityList = this.idenCommunityService.list();
        if(CollUtil.isEmpty(idenCommunityList)){
            return;
        }
        for (IdenCommunity idenCommunity : idenCommunityList) {
            Long communityId = idenCommunity.getId();
            //遍历所有陌生人
            List<IdenPersonStranger>  idenPersonStrangerList = this.idenPersonStrangerService.list();
            if(CollUtil.isEmpty(idenPersonStrangerList)){
                return;
            }

            for (IdenPersonStranger idenPersonStranger : idenPersonStrangerList) {

                //查询此人1月内在该小区出现天次
                QueryWrapper<IdenFaceImage> queryWrapper = new QueryWrapper<>();
                queryWrapper.select("distinct data_date dataDate ") ;
                queryWrapper.lambda().eq(IdenFaceImage::getUid,idenPersonStranger.getUid()).eq(IdenFaceImage::getCommunityId,communityId)
                        .orderByAsc(IdenFaceImage::getDataDate);
                queryWrapper.apply(" data_date >= date_format(DATE_SUB(CURDATE(), INTERVAL 30 DAY),'%Y%m%d')  and  data_date <= date_format(DATE_SUB(CURDATE(), INTERVAL 1 DAY),'%Y%m%d')");

                int strangerDayFrequency = 0;
                //1月内7天出现情况
                List<IdenFaceImage> listDataDate =  this.idenFaceImageService.list(queryWrapper);
                if(listDataDate == null){
                    continue;
                } else {
                    strangerDayFrequency = listDataDate.size();
                }

                QueryWrapper<IdenWarningRuleStranger> queryWrapper3 = new QueryWrapper<>();
                IdenWarningRuleStranger idenWarningRuleStranger = idenWarningRuleStrangerService.getOne(queryWrapper3);
                int strangerFrequencyRule = 15;
                int strangerDays = 7;
                if (idenWarningRuleStranger != null){
                    strangerFrequencyRule = idenWarningRuleStranger.getStrangerFrequency();
                    strangerDays = idenWarningRuleStranger.getStrangerDays();
                }
                String isContinue = getIsContinue(listDataDate,strangerDays);

                boolean isWarning = false;
                String strangerType = null;

                //此人1月出现频次少于15次及7内天每天未连续出现,视为陌生人
                if(strangerDayFrequency < strangerFrequencyRule &&  IsContinueEnum.NO.getValue().equals(isContinue)) {
                    isWarning = true;
                    strangerType = StrangerTypeEnum.STRANGER.getValue();
                    //此人出现频次1月内大于15天次或连续7天每天都出现，标注为疑似未登记住户
                } else if(strangerDayFrequency >= strangerFrequencyRule ||  IsContinueEnum.YES.getValue().equals(isContinue)) {
                    isWarning = true;
                    strangerType = StrangerTypeEnum.NO_REGISTER.getValue();
                }
                if(isWarning) {

                    QueryWrapper<IdenWarningEvent> idenWarningEventQueryWrapper = new QueryWrapper<>();
                    idenWarningEventQueryWrapper.lambda()
                            .eq(IdenWarningEvent::getCommunityId,communityId)
                            .eq(IdenWarningEvent::getUid,idenPersonStranger.getUid())
                            .eq(IdenWarningEvent::getType,WarningTypeEnum.STRANGER.getValue());
                    idenWarningEventQueryWrapper.apply("date_format(iden_warning_event.create_time,'%Y-%m-%d') = date_format(now(),'%Y-%m-%d')");

                   if(this.idenWarningEventService.count(idenWarningEventQueryWrapper) > 0) { //今天数据已经处理过了
                       continue;
                   }

                    //查询是不是已经有预警人员数据了
                    QueryWrapper<IdenWarningPerson> queryWrapper0 = new QueryWrapper<>();
                    queryWrapper0.lambda()
                            .eq(IdenWarningPerson::getCommunityId,communityId)
                            .eq(IdenWarningPerson::getUid,idenPersonStranger.getUid())
                            .eq(IdenWarningPerson::getType,WarningTypeEnum.STRANGER.getValue());
                    IdenWarningPerson idenWarningPerson = this.idenWarningPersonService.getOne(queryWrapper0);

                    if (idenWarningPerson != null) {

                        idenWarningPerson.setAppearFrequency(strangerDayFrequency);
                        idenWarningPerson.setStrangerType(strangerType);
                        idenWarningPerson.setHandleStatus(HandleWarningStatusEnum.NO_HANDLE.getValue());
                        idenWarningPerson.setModifyTime(new Date());
                        this.idenWarningPersonService.updateById(idenWarningPerson);
                    } else {
                        idenWarningPerson = new IdenWarningPerson();
                        idenWarningPerson.setType(WarningTypeEnum.STRANGER.getValue());
                        idenWarningPerson.setStrangerType(strangerType);
                        idenWarningPerson.setUid(idenPersonStranger.getUid());
                        idenWarningPerson.setDataDate(DateUtils.getYestodayYyyyMMddDate());
                        idenWarningPerson.setImage(idenPersonStranger.getImage());
                        idenWarningPerson.setFeatPtr(idenPersonStranger.getFeatPtr());

                        idenWarningPerson.setCommunityId(communityId);
                        idenWarningPerson.setCommunityCode(idenCommunity.getCode());
                        idenWarningPerson.setCommunityName(idenCommunity.getName());
                        idenWarningPerson.setSubdistrict(idenCommunity.getSubdistrict());
                        idenWarningPerson.setDistrict(idenCommunity.getDistrict());

                        idenWarningPerson.setAppearFrequency(strangerDayFrequency);
                        idenWarningPerson.setIsContinue(isContinue);

                        //查询此人在该小区最早出现时间
                        QueryWrapper<IdenFaceImage> queryWrapper1 = new QueryWrapper<>();
                        queryWrapper1.lambda().eq(IdenFaceImage::getCommunityId,communityId).eq(IdenFaceImage::getUid,idenPersonStranger.getUid())
                                .orderByAsc(IdenFaceImage::getPhotographTime);
                        queryWrapper1.last("limit 1");

                        IdenFaceImage idenFaceImage1 = this.idenFaceImageService.getOne(queryWrapper1);
                        if(idenFaceImage1 != null){
                            idenWarningPerson.setEarliestAppearTime(idenFaceImage1.getPhotographTime());
                        }

                        idenWarningPerson.setCreateTime(new Date());
                        idenWarningPerson.setModifyTime(new Date());
                        this.idenWarningPersonService.save(idenWarningPerson);

                    }

                    IdenWarningPersonHis idenWarningPersonHis = new IdenWarningPersonHis();
                    BeanUtil.copyProperties(idenWarningPerson,idenWarningPersonHis,"id");
                    idenWarningPersonHis.setCreateTime(new Date());
                    idenWarningPersonHis.setModifyTime(new Date());
                    idenWarningPersonHisService.save(idenWarningPersonHis);

                    IdenWarningEvent idenWarningEvent = new IdenWarningEvent();
                    idenWarningEvent.setWarningPersonId(idenWarningPerson.getId());
                    idenWarningEvent.setUid(idenWarningPerson.getUid());
                    idenWarningEvent.setPersonId(idenWarningPerson.getPersonId());

                    //查询此人在该小区最晚出现时间
                    QueryWrapper<IdenFaceImage> queryWrapper2 = new QueryWrapper<>();
                    queryWrapper2.lambda().eq(IdenFaceImage::getCommunityId,communityId).eq(IdenFaceImage::getUid,idenPersonStranger.getUid())
                            .orderByDesc(IdenFaceImage::getPhotographTime);
                    queryWrapper2.last("limit 1");
                    IdenFaceImage idenFaceImage2 = this.idenFaceImageService.getOne(queryWrapper2);
                    if(idenFaceImage2 != null){
                        idenWarningEvent.setDataTime(idenFaceImage2.getPhotographTime());
                        idenWarningEvent.setImage(idenFaceImage2.getImage());
                        idenWarningEvent.setAppearTime(idenFaceImage2.getPhotographTime());
                    }

                    idenWarningEvent.setCommunityId(idenWarningPerson.getCommunityId());
                    idenWarningEvent.setCommunityCode(idenWarningPerson.getCommunityCode());
                    idenWarningEvent.setCommunityName(idenWarningPerson.getCommunityName());
                    idenWarningEvent.setDistrict(idenWarningPerson.getDistrict());
                    idenWarningEvent.setSubdistrict(idenWarningPerson.getSubdistrict());
                    idenWarningEvent.setType(WarningTypeEnum.STRANGER.getValue());
                    idenWarningEvent.setCreateTime(new Date());
                    idenWarningEvent.setModifyTime(new Date());
                    this.idenWarningEventService.save(idenWarningEvent);
                }

            }

        }

    }


    //常住人员识别预警,小区人脸库内7天内未被命中人员或标记为独居老人或老人带小孩的7天内未被命中人员视为预警对象
    private void handleResident() {

        //查询全部小区人员
        QueryWrapper<IdenPerson> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().like(IdenPerson::getType, PersonTypeEnum.COMMUNITY.getValue());
        List<IdenPerson> listPerson = this.idenPersonService.list(queryWrapper);
        if(CollUtil.isEmpty(listPerson)){
            return;
        }
        //遍历小区人员
        for (IdenPerson idenPerson :listPerson ) {
            QueryWrapper<IdenFaceImage> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.lambda().eq(IdenFaceImage::getPersonId, idenPerson.getId())
                    .orderByDesc(IdenFaceImage::getPhotographTime);
            queryWrapper1.last("limit 1"); //查询最后一次拍照

            IdenFaceImage idenFaceImage = this.idenFaceImageService.getOne(queryWrapper1);

            int continueDisappearDaysRule = 7;
            QueryWrapper<IdenWarningRuleResident> queryWrapper2 = new QueryWrapper<>();
            IdenWarningRuleResident idenWarningRuleResident = this.idenWarningRuleResidentService.getOne(queryWrapper2);
            if(idenWarningRuleResident != null) {
                continueDisappearDaysRule = idenWarningRuleResident.getContinueDisappearDays();
            }

            if (idenFaceImage == null){
                idenFaceImage = new IdenFaceImage();
                idenFaceImage.setPhotographTime(DateUtils.getBeforeDates(100)); //设定一个假时间
            }
            //最后一次拍照时间和昨天的相差几天
            int continueDisappearDays = DateUtils.getDaysGap(idenFaceImage.getPhotographTime(), new Date()) - 1;
            //超过7天没出现, 满足预警
            if ( continueDisappearDays > continueDisappearDaysRule ) {

                QueryWrapper<IdenWarningEvent> idenWarningEventQueryWrapper = new QueryWrapper<>();
                idenWarningEventQueryWrapper.lambda()
                        .eq(IdenWarningEvent::getCommunityId,idenPerson.getCommunityId())
                        .eq(IdenWarningEvent::getUid,idenPerson.getUid())
                        .eq(IdenWarningEvent::getType,WarningTypeEnum.RESIDENT.getValue());
                idenWarningEventQueryWrapper.apply("date_format(iden_warning_event.create_time,'%Y-%m-%d') = date_format(now(),'%Y-%m-%d')");

                if(this.idenWarningEventService.count(idenWarningEventQueryWrapper) > 0) { //今天数据已经处理过了
                    continue;
                }

                //查询是不是已经有预警人员数据了
                QueryWrapper<IdenWarningPerson> queryWrapper0 = new QueryWrapper<>();
                queryWrapper0.lambda()
                        .eq(IdenWarningPerson::getCommunityId,idenPerson.getCommunityId())
                        .eq(IdenWarningPerson::getUid,idenPerson.getUid())
                        .eq(IdenWarningPerson::getType,WarningTypeEnum.RESIDENT.getValue());
                IdenWarningPerson idenWarningPerson = this.idenWarningPersonService.getOne(queryWrapper0);

                if (idenWarningPerson != null) {
                    idenWarningPerson.setContinueDisappearDays(continueDisappearDays);
                    idenWarningPerson.setModifyTime(new Date());
                    idenWarningPerson.setHandleStatus(HandleWarningStatusEnum.NO_HANDLE.getValue());
                    this.idenWarningPersonService.updateById(idenWarningPerson);
                } else {
                    //生成预警数据
                    idenWarningPerson = new IdenWarningPerson();
                    idenWarningPerson.setUid(idenPerson.getUid());
                    idenWarningPerson.setPersonId(idenPerson.getId());
                    idenWarningPerson.setPersonName(idenPerson.getName());
                    idenWarningPerson.setImage(idenPerson.getImage());
                    idenWarningPerson.setFeatPtr(idenPerson.getFeatPtr());
                    idenWarningPerson.setDataDate(DateUtils.getYestodayYyyyMMddDate());
                    Long communityId = idenPerson.getCommunityId();
                    IdenCommunity idenCommunity = this.idenCommunityService.getById(communityId);
                    if (idenCommunity != null) {
                        idenWarningPerson.setCommunityId(communityId);
                        idenWarningPerson.setCommunityCode(idenCommunity.getCode());
                        idenWarningPerson.setCommunityName(idenCommunity.getName());
                        idenWarningPerson.setSubdistrict(idenCommunity.getSubdistrict());
                        idenWarningPerson.setDistrict(idenCommunity.getDistrict());
                    }

                    idenWarningPerson.setContinueDisappearDays(continueDisappearDays);
                    //此人在该小区最晚出现时间
                    idenWarningPerson.setLatestAppearTime(idenFaceImage.getPhotographTime());

                    idenWarningPerson.setType(WarningTypeEnum.RESIDENT.getValue());
                    idenWarningPerson.setCreateTime(new Date());
                    idenWarningPerson.setModifyTime(new Date());
                    this.idenWarningPersonService.save(idenWarningPerson);
                }

                IdenWarningPersonHis idenWarningPersonHis = new IdenWarningPersonHis();
                BeanUtil.copyProperties(idenWarningPerson,idenWarningPersonHis,"id");
                idenWarningPersonHis.setCreateTime(new Date());
                idenWarningPersonHis.setModifyTime(new Date());
                idenWarningPersonHisService.save(idenWarningPersonHis);

                IdenWarningEvent idenWarningEvent = new IdenWarningEvent();
                idenWarningEvent.setWarningPersonId(idenWarningPerson.getId());
                idenWarningEvent.setUid(idenWarningPerson.getUid());
                idenWarningEvent.setPersonId(idenWarningPerson.getPersonId());

                idenWarningEvent.setDataTime(idenWarningPerson.getLatestAppearTime());
                idenWarningEvent.setImage(idenWarningPerson.getImage());
                idenWarningEvent.setCommunityId(idenWarningPerson.getCommunityId());
                idenWarningEvent.setCommunityCode(idenWarningPerson.getCommunityCode());
                idenWarningEvent.setCommunityName(idenWarningPerson.getCommunityName());
                idenWarningEvent.setDistrict(idenWarningPerson.getDistrict());
                idenWarningEvent.setSubdistrict(idenWarningPerson.getSubdistrict());
                idenWarningEvent.setAppearTime(idenWarningPerson.getLatestAppearTime());
                idenWarningEvent.setType(WarningTypeEnum.RESIDENT.getValue());

                idenWarningEvent.setCreateTime(new Date());
                idenWarningEvent.setModifyTime(new Date());
                this.idenWarningEventService.save(idenWarningEvent);
            }
        }

    }

    //昼伏夜出人员识别预警
    private void handleEveningComming() {

        List<IdenCommunity>  idenCommunityList = this.idenCommunityService.list();
        if(CollUtil.isEmpty(idenCommunityList)){
            return;
        }

        //设置默认
        List<IdenWarningRuleEveningComming> idenWarningRuleEveningCommings = new ArrayList<>();
        IdenWarningRuleEveningComming idenWarningRuleEveningComming = new IdenWarningRuleEveningComming();
        idenWarningRuleEveningComming.setStartTime("23:00:00");
        idenWarningRuleEveningComming.setEndTime("04:00:00");
        idenWarningRuleEveningComming.setStatus(AppearStatusEnum.APPEAR.getValue());
        idenWarningRuleEveningComming.setContinueDays(7);
        idenWarningRuleEveningCommings.add(idenWarningRuleEveningComming);

        idenWarningRuleEveningComming.setStartTime("08:00:00");
        idenWarningRuleEveningComming.setEndTime("23:00:00");
        idenWarningRuleEveningComming.setStatus(AppearStatusEnum.DISAPPEAR.getValue());
        idenWarningRuleEveningComming.setContinueDays(7);
        idenWarningRuleEveningCommings.add(idenWarningRuleEveningComming);

        //查询数据库配置，若设置了值，则替换默认
        QueryWrapper<IdenWarningRuleEveningComming> queryWrapper3 = new QueryWrapper<>();
        List<IdenWarningRuleEveningComming> idenWarningRuleEveningCommingsDb = idenWarningRuleEveningCommingService.list(queryWrapper3);
        if (CollUtil.isNotEmpty(idenWarningRuleEveningCommingsDb)) {
            idenWarningRuleEveningCommings = idenWarningRuleEveningCommingsDb;
        }

        //持续多少天
        int continueDays = idenWarningRuleEveningCommings.get(0).getContinueDays();
        //n天前
        String beforeDate = DateUtils.formatToDateStr(DateUtils.getBeforeDates(continueDays + 1),"yyyyMMdd");

        String yestoday = DateUtils.getYestodayYyyyMMddDate();

        for (IdenCommunity idenCommunity : idenCommunityList) {

            //查询全部小区人员
            QueryWrapper<IdenPerson> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().like(IdenPerson::getType, PersonTypeEnum.COMMUNITY.getValue());
            List<IdenPerson> listPerson = this.idenPersonService.list(queryWrapper);
            if(CollUtil.isNotEmpty(listPerson)){
                //遍历小区人员
                for (IdenPerson idenPerson :listPerson ) {
                    handleEveningCommingSub(continueDays,beforeDate,yestoday,idenPerson.getUid(),
                            idenCommunity,idenWarningRuleEveningCommings,idenPerson.getId(),idenPerson.getImage(),idenPerson.getFeatPtr());
                }
            }

            //遍历所有陌生人
            List<IdenPersonStranger>  idenPersonStrangerList = this.idenPersonStrangerService.list();
            if(CollUtil.isNotEmpty(idenPersonStrangerList)){
                for (IdenPersonStranger idenPersonStranger : idenPersonStrangerList ) {
                    handleEveningCommingSub(continueDays,beforeDate,yestoday,idenPersonStranger.getUid(),
                            idenCommunity,idenWarningRuleEveningCommings,null,idenPersonStranger.getImage(),idenPersonStranger.getFeatPtr());
                }
            }

        }

    }

    private void handleEveningCommingSub(int continueDays,String beforeDate,String yestoday,Long uid,
                                         IdenCommunity idenCommunity, List<IdenWarningRuleEveningComming>  idenWarningRuleEveningCommings,
                                         Long personId,String image,String featPtr){
        Long communityId = idenCommunity.getId();
        //查询此人长期晚上某时段出现（时间期间可设）且白天未出现，视为昼伏夜出人员，进行预警
        QueryWrapper<IdenFaceImage> queryWrapper = new QueryWrapper<>();
        queryWrapper.select("distinct data_date dataDate ") ;
        queryWrapper.lambda().eq(IdenFaceImage::getUid,uid)
                .eq(IdenFaceImage::getCommunityId,communityId)
                .ge(IdenFaceImage::getDataDate,beforeDate)
                .le(IdenFaceImage::getDataDate,yestoday);

        for(IdenWarningRuleEveningComming idenWarningRuleEveningComming1 : idenWarningRuleEveningCommings) {
            String startTime = idenWarningRuleEveningComming1.getStartTime();
            String endTime = idenWarningRuleEveningComming1.getEndTime();
            Map<String, String> map = DateUtils.convertDate(startTime,endTime);
            String startDate = map.get("startDate");
            String endDate = map.get("endDate");
            if (AppearStatusEnum.APPEAR.getValue().equals(idenWarningRuleEveningComming1.getStatus())) {
                queryWrapper.apply(" photograph_time > '" + startDate + "' and  photograph_time < '" + endDate + "'");
            } else if (AppearStatusEnum.DISAPPEAR.getValue().equals(idenWarningRuleEveningComming1.getStatus())) {
                queryWrapper.apply(" not exists (select 1 from iden_face_image a where a.data_date = iden_face_image.data_date" +
                        " and a.photograph_time > '" + startDate + "' and a.photograph_time <  '" + endDate +  "') ");
            }
        }

        int count = 0;
        List<IdenFaceImage> idenFaceImageList = this.idenFaceImageService.list(queryWrapper);
        if(idenFaceImageList == null){
           return;
        } else {
            count = idenFaceImageList.size();
        }
        if (count >= continueDays) { //连续超过7天，满足预警

            QueryWrapper<IdenWarningEvent> idenWarningEventQueryWrapper = new QueryWrapper<>();
            idenWarningEventQueryWrapper.lambda()
                    .eq(IdenWarningEvent::getCommunityId,communityId)
                    .eq(IdenWarningEvent::getUid,uid)
                    .eq(IdenWarningEvent::getType,WarningTypeEnum.EVENING_COMMING.getValue());
            idenWarningEventQueryWrapper.apply("date_format(iden_warning_event.create_time,'%Y-%m-%d') = date_format(now(),'%Y-%m-%d')");

            if(this.idenWarningEventService.count(idenWarningEventQueryWrapper) > 0) { //今天数据已经处理过了
                return;
            }

            //查询是不是已经有预警人员数据了
            QueryWrapper<IdenWarningPerson> queryWrapper0 = new QueryWrapper<>();
            queryWrapper0.lambda()
                    .eq(IdenWarningPerson::getCommunityId,communityId)
                    .eq(IdenWarningPerson::getUid,uid)
                    .eq(IdenWarningPerson::getType,WarningTypeEnum.EVENING_COMMING.getValue());
            IdenWarningPerson idenWarningPerson = this.idenWarningPersonService.getOne(queryWrapper0);
            if (idenWarningPerson != null) {

                idenWarningPerson.setAppearFrequency(count);
                idenWarningPerson.setHandleStatus(HandleWarningStatusEnum.NO_HANDLE.getValue());
                idenWarningPerson.setModifyTime(new Date());
                this.idenWarningPersonService.updateById(idenWarningPerson);

            } else {
                idenWarningPerson = new IdenWarningPerson();
                idenWarningPerson.setUid(uid);
                idenWarningPerson.setPersonId(personId);
                if(idenWarningPerson.getPersonId() != null){
                    IdenPerson idenPerson = this.idenPersonService.getById(idenWarningPerson.getPersonId());
                    if(idenPerson != null){
                        idenWarningPerson.setPersonName(idenPerson.getName());
                    }
                }
                idenWarningPerson.setDataDate(DateUtils.getYestodayYyyyMMddDate());
                idenWarningPerson.setImage(image);
                idenWarningPerson.setFeatPtr(featPtr);

                idenWarningPerson.setDistrict(idenCommunity.getDistrict());
                idenWarningPerson.setSubdistrict(idenCommunity.getSubdistrict());

                idenWarningPerson.setCommunityId(communityId);
                idenWarningPerson.setCommunityCode(idenCommunity.getCode());
                idenWarningPerson.setCommunityName(idenCommunity.getName());

                idenWarningPerson.setAppearFrequency(count);

                //查询此人在该小区最早出现时间
                QueryWrapper<IdenFaceImage> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.lambda().eq(IdenFaceImage::getCommunityId,communityId).eq(IdenFaceImage::getUid,uid)
                        .orderByAsc(IdenFaceImage::getPhotographTime);
                queryWrapper1.last("limit 1");
                IdenFaceImage idenFaceImage1 = this.idenFaceImageService.getOne(queryWrapper1);
                if(idenFaceImage1 != null){
                    idenWarningPerson.setEarliestAppearTime(idenFaceImage1.getPhotographTime());
                }

                idenWarningPerson.setType(WarningTypeEnum.EVENING_COMMING.getValue());
                idenWarningPerson.setCreateTime(new Date());
                idenWarningPerson.setModifyTime(new Date());
                this.idenWarningPersonService.save(idenWarningPerson);

            }
            IdenWarningPersonHis idenWarningPersonHis = new IdenWarningPersonHis();
            BeanUtil.copyProperties(idenWarningPerson,idenWarningPersonHis,"id");
            idenWarningPersonHis.setCreateTime(new Date());
            idenWarningPersonHis.setModifyTime(new Date());
            idenWarningPersonHisService.save(idenWarningPersonHis);

            IdenWarningEvent idenWarningEvent = new IdenWarningEvent();
            idenWarningEvent.setWarningPersonId(idenWarningPerson.getId());
            idenWarningEvent.setUid(idenWarningPerson.getUid());
            idenWarningEvent.setPersonId(idenWarningPerson.getPersonId());

            QueryWrapper<IdenFaceImage> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.lambda().eq(IdenFaceImage::getUid, uid).eq(IdenFaceImage::getCommunityId,communityId)
                    .orderByDesc(IdenFaceImage::getPhotographTime);
            queryWrapper1.last("limit 1"); //查询最后一次拍照

            IdenFaceImage idenFaceImage = this.idenFaceImageService.getOne(queryWrapper1);

            idenWarningEvent.setDataTime(idenFaceImage.getPhotographTime());
            idenWarningEvent.setImage(idenFaceImage.getImage());
            idenWarningEvent.setCommunityId(idenWarningPerson.getCommunityId());
            idenWarningEvent.setCommunityCode(idenWarningPerson.getCommunityCode());
            idenWarningEvent.setCommunityName(idenWarningPerson.getCommunityName());
            idenWarningEvent.setDistrict(idenWarningPerson.getDistrict());
            idenWarningEvent.setSubdistrict(idenWarningPerson.getSubdistrict());
            idenWarningEvent.setAppearTime(idenFaceImage.getPhotographTime());
            idenWarningEvent.setType(WarningTypeEnum.EVENING_COMMING.getValue());
            idenWarningEvent.setCreateTime(new Date());
            idenWarningEvent.setModifyTime(new Date());
            this.idenWarningEventService.save(idenWarningEvent);
        }
    }

    //按小区统计
    private void staCommunity(int staBeforeDayNums) {
        idenWarningStaService.insertWarningPersonRateSta(staBeforeDayNums);
    }

    //是否有连续7天情况
    private String getIsContinue(List<IdenFaceImage> listDataDate,int strangerDays) {
        if(CollUtil.isEmpty(listDataDate)){
            return IsContinueEnum.NO.getValue();
        }
        List<String> dataDateList = new ArrayList<>();

       for(IdenFaceImage idenFaceImage : listDataDate) {
           dataDateList.add(idenFaceImage.getDataDate());
       }

        String minDataDate = listDataDate.get(0).getDataDate();
        String maxDataDate = listDataDate.get(listDataDate.size() - 1).getDataDate();

        List<String> list =  DateUtils.getBetweenDates(minDataDate,maxDataDate);
        int cnt = 0;
        for (String date : list) {
            if(dataDateList.contains(date)){
                cnt++;
            } else {
                cnt = 0;
            }
            if(cnt >= strangerDays) {
                return IsContinueEnum.YES.getValue();
            }
        }

        return IsContinueEnum.NO.getValue();
    }
}